;;; $DOOMDIR/config.el -*- lexical-binding: t; -*-

(setq user-full-name "Tim van Deurzen"
      user-mail-address "tim@kompiler.org")

;;; UI
(setq doom-theme 'doom-one)

(setq display-line-numbers-type 'relative)

(setq autosave-defaults t
      make-backup-files t)

(require 'which-key)
(setq which-key-idle-delay 0.1)

(setq confirm-kill-emacs nil)

(use-package! evil-colemak-basics
  :after evil
  :hook (after-init . global-evil-colemak-basics-mode)
  :config
  (setq evil-colemak-basics-rotate-t-f-j t))

(after! evil (global-evil-colemak-basics-mode t))

(setq default-frame-alist
      (append (list
               '(right-divider-width . 1)
               '(bottom-divider-width . 1)
               '(vertical-scroll-bars . nil)
               '(internal-border-width . 15)
               '(left-fringe . 4)
               '(right-fringe . 4)
               '(tool-bar-lines . 0)
               '(menu-bar-lines . 0)
               '(buffer-predicate . doom-buffer-frame-predicate))))


;;; ORG

(use-package! org-appear
  :after org
  :hook (org-mode . org-appear-mode)
  :config
  (setq org-appear-autolinks t
        org-appear-autoentities t
        org-appear-autosubmarkers t))

(after! org
  (setq org-directory "~/org/")

  (setq org-todo-keywords
        '((sequence "TODO(t)" "NEXT(n!)" "|" "DONE(d!)")
          (sequence "WAITING(w@/!)" "SOMEDAY(s)" "|" "CANCELED(c@)")))

  (setq org-hide-emphasis-markers t
        org-startup-folded 'overview
        org-pretty-entities t

        org-time-clocksum-format (quote (:hours "%d" :require-hours t :minutes ":%02d" :require-minutes t))
        org-duration-format (quote h:mm)
        org-agenda-format-date (lambda (date) (concat "\n"
                                                      (make-string (window-width) 9472)
                                                      "\n"
                                                      (org-agenda-format-date-aligned date)))

        org-log-done 'time
        org-log-into-drawer t

        org-agenda-start-with-log-mode t
        org-agenda-skip-deadline-if-done t
        org-agenda-skip-scheduled-if-done t

        org-clock-rounding-minutes 15
        org-clock-persist t
        org-clock-in-resume t
        org-clock-persist-query-resume nil
        org-clock-into-drawer t
        org-clock-out-remove-zero-time-clocks t
        org-clock-report-include-clocking-task t

        org-habit-show-habits t
        org-habit-following-days 7
        org-habit-preceding-days 21
        org-habit-show-habits-only-for-today t

        org-enforce-todo-dependencies t
        org-enforce-todo-checkbox-dependencies t
        alert-default-style 'libnotify
        org-catch-invisible-edits 'show-and-error

        org-refile-use-outline-path 'full-file-path
        org-outline-path-complete-in-steps nil
        org-refile-allow-creating-parent-nodes 'confirm
        org-refile-targets '((nil :maxlevel . 3)
                             (org-agenda-files :maxlevel . 3))

        org-id-track-globally t))

(add-hook! org-mode :append
           #'visual-line-mode
           #'org-appear-mode)

(use-package! org-transclusion
  :after org-roam)

(after! org-roam
  (setq org-roam-directory "~/org/roam/"
        org-roam-v2-ack t
        org-roam-db-update-on-save t)

  (setq deft-directory "~/org/roam")
  (setq deft-extensions '("org"))

  (org-roam-db-autosync-mode))


(require 'vulpea-select)
(defun tim/org-roam-filter-by-tag (tag)
  "Return a lambda to select nodes with a particular tag TAG."

  (lambda (node)
    (member tag (org-roam-node-tags node))))

(defun tim/hidden-file-p (name)
  "Return whether the file with name NAME is hidden."

  (string-prefix-p "." name))

(defun tim/org-roam-workspaces ()
  "Use subfolders in the org-roam-directory ask workspaces for particular projects."

  (let ((files (directory-files org-roam-directory)))
    (seq-filter (lambda (name)
                  (and (file-directory-p (concat org-roam-directory name))
                       (not (tim/hidden-file-p name))))
                files)))


(defun tim/find-project ()
  "Select or capture project within a particular workspace."

  (interactive)

  (let* ((workspace (completing-read "workspace: " (tim/org-roam-workspaces)))
         (node (vulpea-select-from "item" (vulpea-db-query-by-tags-every (list workspace "project"))))
         (org-roam-directory (expand-file-name workspace org-roam-directory)))

    (if (vulpea-note-id node)
        (org-roam-node-visit (org-roam-node-from-id (vulpea-note-id node)))
      (org-roam-capture- :node (org-roam-node-create :title (vulpea-note-title node))
                         :props '(:finalize find-file)
                         :templates
                         `(
                           ("p" "project" plain (file ,(concat org-roam-directory "/templates/project.org"))
                            :target (file+head "%<%Y%m%d%H%M%S>-${slug}.org" "#+title: ${title}")
                            :unnarrowed t)

                           ("n" "note" plain (file ,(concat org-roam-directory "/templates/note.org"))
                            :target (file+head "%<%Y%m%d%H%M%S>-${slug}.org" "#+title: ${title}")
                            :unnarrowed t)
                           )))))

;; Yes this is duplicated ... still learning elisp.
(defun tim/find-report ()
  "Select or capture report within a particular workspace."

  (interactive)

  (let* ((workspace (completing-read "workspace: " (tim/org-roam-workspaces)))
         (node (vulpea-select-from "report" (vulpea-db-query-by-tags-every (list workspace "report"))))
         (org-roam-directory (expand-file-name workspace org-roam-directory)))

    (if (vulpea-note-id node)
        (org-roam-node-visit (org-roam-node-from-id (vulpea-note-id node)))
      (org-roam-capture- :node (org-roam-node-create :title (vulpea-note-title node))
                         :props '(:finalize find-file)
                         :templates
                         `(
                           ("m" "monthly report" plain (file ,(concat org-roam-directory "/templates/monthly-report.org"))
                            :target (file+head "%<%Y-%B>.org" "#+title: ${title}")
                            :unnarrowed t)

                           ("y" "yearly report" plain (file ,(concat org-roam-directory "/templates/yearly-report.org"))
                            :target (file+head "%<%Y>.org" "#+title: ${title}")
                            :unnarrowed t)

                           )))))

(map! :leader
      (:prefix-map ("n" . "notes")
       (:prefix ("p" . "personal workflow")
        :desc "Find project in a workspace." "f" #'tim/find-project
        :desc "Find report in a workspace." "r" #'tim/find-report)))


;; (defun tim/org-roam-find-project-workspace ()
;;   (interactive)
;;
;;   (let* ((directory (completing-read "workspace: " (ayrat555/org-roam-directories)))
;;          (org-roam-directory (expand-file-name directory org-roam-directory)))
;;     (org-roam-node-find
;;      nil
;;      nil
;;      (tim/org-roam-filter-by-tag directory))
;;     ))

;; TODO Define custom key maps to load a specific agenda view that only contains relevant files.
;; TODO Redefine the default agenda to use a static set of files?
(use-package! org-super-agenda
  :after org-agenda
  :hook (org-agenda-mode . org-super-agenda-mode)
  :config
  (setq org-agenda-custom-commands
        '(("z" "Custom Day"
           ((agenda "" ((org-agenda-span 'day)
                        (org-super-agenda-groups
                         '((:log t)
                           (:name "Schedule"
                            :time-grid t)
                           (:habit t)
                           (:name "Scheduled future"
                            :scheduled future)
                           (:name "Scheduled past"
                            :scheduled past)
                           (:name "Due today"
                            :deadline today)
                           (:name "Overdue"
                            :deadline past)
                           (:name "Due soon"
                            :deadline future)
                           (:name "Waiting"
                            :todo "WAITING"
                            :order 10)
                           ))))))
          ("w" "Custom Week"
           ((agenda "" ((org-agenda-span 'week)
                        (org-super-agenda-groups
                         '((:log t)
                           (:name "Schedule"
                            :time-grid t)
                           (:name "Planning"
                            :scheduled t)
                           (:name "Due"
                            :deadline t)
                           (:name "Overdue"
                            :deadline past)
                           ))))))
          )))


(define-derived-mode project-explore-mode tabulated-list-mode "project-explore-mode"
  "Major mode to explore projects."

  (setq tabulated-list-format [("Title" 20 t)
                               ("Summary" 60 nil)
                               ("Date" 11 t)
                               ("Tags" 10 nil)])
  (setq tabulated-list-padding 2)
  (setq tabulated-list-sort-key (cons "Date" t))
  (use-local-map project-explore-mode-map)
  (tabulated-list-init-header))

(defun tim/my-projects ()
  "List all projects."

  (mapcar
   (lambda (node) (list
                   (org-roam-node-id node)
                   (vector
                    (org-roam-node-title node)
                    (org-roam-node-file node)
                    "date"
                    (org-roam-node-tags node))))
   (seq-filter (tim/org-roam-filter-by-tag "bux") (org-roam-node-list))))

(defun explore-projects ()
  (interactive)
  (pop-to-buffer "*Project Explore Mode*" nil)
  (project-explore-mode)
  (setq tabulated-list-entries (tim/my-projects))
  (tabulated-list-print t))

(use-package! vulpea
  :config (vulpea-db-autosync-mode))

;; Here are some additional functions/macros that could help you configure Doom:
;;
;; - `load!' for loading external *.el files relative to this one
;; - `use-package!' for configuring packages
;; - `after!' for running code after a package has loaded
;; - `add-load-path!' for adding directories to the `load-path', relative to
;;   this file. Emacs searches the `load-path' when you load packages with
;;   `require' or `use-package'.
;; - `map!' for binding new keys
;;
;; To get information about any of these functions/macros, move the cursor over
;; the highlighted symbol at press 'K' (non-evil users must press 'C-c c k').
;; This will open documentation for it, including demos of how they are used.
;;
;; You can also try 'gd' (or 'C-c c d') to jump to their definition and see how
;; they are implemented.
