set -g theme_display_git yes
set -g theme_display_git_untracked no
set -g theme_display_git_ahead_verbose yes
set -g theme_git_worktree_support yes
set -g theme_display_vagrant no
set -g theme_display_docker_machine no
set -g theme_display_hg yes
set -g theme_display_virtualenv yes
set -g theme_display_ruby no
set -g theme_display_user yes
set -g theme_display_vi yes
set -g theme_display_date yes
set -g theme_display_cmd_duration yes
set -g theme_title_display_process no
set -g theme_title_display_path no
set -g theme_title_use_abbreviated_path yes
set -g theme_date_format "+%a %H:%M"
set -g theme_avoid_ambiguous_glyphs yes
set -g theme_powerline_fonts yes
set -g theme_nerd_fonts yes
set -g theme_show_exit_status yes
set -g default_user your_normal_user
set -g fish_prompt_pwd_dir_length 0
set -g theme_project_dir_length 1
set -g XDG_CURRENT_DESKTOP GNOME
set -Ux EDITOR nvim

set GEM_ROOT ~/.gem/ruby/2.7.0

# set the workspace path
set -x GOPATH /home/tim/src/golang

# add the go bin path to be able to execute our programs
set -x PATH $GOPATH/bin $GEM_ROOT/bin $HOME/.local/bin $HOME/.cargo/bin $PATH 

# content has to be in .config/fish/config.fish
# if it does not exist, create the file
setenv SSH_ENV $HOME/.ssh/environment

function start_agent
    echo "Initializing new SSH agent ..."
    ssh-agent -c | sed 's/^echo/#echo/' > $SSH_ENV
    echo "succeeded"
    chmod 600 $SSH_ENV
    . $SSH_ENV > /dev/null
    ssh-add
end

function test_identities
    ssh-add -l | grep "The agent has no identities" > /dev/null
    if [ $status -eq 0 ]
        ssh-add
        if [ $status -eq 2 ]
            start_agent
        end
    end
end

if [ -n "$SSH_AGENT_PID" ]
    ps -ef | grep $SSH_AGENT_PID | grep ssh-agent > /dev/null
    if [ $status -eq 0 ]
        test_identities
    end
else
    if [ -f $SSH_ENV ]
        . $SSH_ENV > /dev/null
    end
    ps -ef | grep $SSH_AGENT_PID | grep -v grep | grep ssh-agent > /dev/null
    if [ $status -eq 0 ]
        test_identities
    else
        start_agent
    end
end

# The next line updates PATH for the Google Cloud SDK.
if [ -f '/home/tim/downloads/google-cloud-sdk/path.fish.inc' ]; if type source > /dev/null; source '/home/tim/downloads/google-cloud-sdk/path.fish.inc'; else; . '/home/tim/downloads/google-cloud-sdk/path.fish.inc'; end; end

alias ls='/usr/bin/exa'
alias cat='/usr/bin/bat'

direnv hook fish | source

starship init fish | source

zoxide init fish | source
