local function apply_buffer_window_options(options)
	for k, v in pairs(options) do
		if v == true or v == false then
			vim.cmd('set ' .. k)
		else
			vim.cmd('set ' .. k .. '=' .. v)
		end
	end
end

local function basic_configuration()
	local options = {
	    winbar = "%=%m %f";
		hidden = true;
		fileformats = "unix,mac,dos";
		modeline = true;
		modelines = 5;
		whichwrap = "h,l,<,>,[,],~";
		joinspaces = true;
		mouse = "a";
		incsearch = true;
		hlsearch = true;
		ignorecase = true;
		smartcase = true;
		history = 1000;
		undofile = true;
		undodir = "/home/tim/.local/share/nvim/undo";
		undolevels = 1000;
		backspace = "indent,eol,start";
		syntax = "on";
		fillchars = "vert:│";
		termguicolors = true;
		background = "dark";
		ttyfast = true;
		-- lazyredraw = true;
		listchars = "tab:»·,nbsp:+,trail:·,extends:→,precedes:←";
		showcmd = true;
		ruler = true;
		rulerformat = "%l/%L:%c";
		equalalways = true;
		splitbelow = true;
		splitright = true;
		wildmenu = true;
		wildignore = "*.o,*.class,*.hi,*.pdf,*.git,*.blg,*.bbl,*.aux,*.gcno,*.gcda";
		showmode = true;
		laststatus = 3;
		scrolloff = 7;
		cmdheight = 2;
		tabpagemax = 100;
		timeout = true;
		ttimeout = true;
		timeoutlen = 100;
		ttimeoutlen = 10;
		updatetime = 400;
		shiftround = true;
		encoding = "utf-8";
		magic = true;
		completeopt = "menu,menuone,noselect";
		shada = "!,'300,<50,@100,s10,h";
		inccommand = "nosplit";
		grepformat = "%f:%l:%c:%m";
		grepprg = 'rg --hidden --vimgrep --smart-case --';
		switchbuf = "usetab";
		-- diffopt = "filler,iwhite,internal,algorithm:patience";
		--showtabline = true;
		pumheight = 15;
		helpheight = 12;
		previewheight = 12;
		showbreak = "↳  ";
		pumblend = 15;
		winblend = 15;
		-- Sets the keymap in normal mode to QWERTY while actually on Colemak.
		langmap = "rs,RS,gt,GT,fe,FE,pr,PR,jy,JY,lu,LU,ui,UI,yo,YO,sd,SD,dg,DG,nj,NJ,ek,EK,il,IL,kn,KN,o\\;,O:,\\;p";
		langnoremap = true;
	}

	local buffer_window_options = {
		formatoptions = "qroctn";
		shiftwidth = 4;
		tabstop = 4;
		expandtab = true;
		autoindent = true;
		copyindent = true;
		smarttab = true;
		softtabstop = 4;
		number = true;
		relativenumber = true;
		cursorline = true;
		wrap = false;
		signcolumn = "yes";
		textwidth = 90;
	}

	-- global settings
	vim.g.python_host_prog = '/usr/bin/python'
	vim.g.python3_host_prog = '/usr/bin/python3'
	vim.g.mapleader = " "
	vim.api.nvim_set_keymap('n',' ','',{noremap = true})
	vim.api.nvim_set_keymap('x',' ','',{noremap = true})

	for name, value in pairs(options) do
		vim.o[name] = value
	end

	apply_buffer_window_options(buffer_window_options)
end

basic_configuration()
vim.g.lua_ftplugin = 1
