local lsp_config = require("lspconfig")
-- local ih = require("inlay-hints")
--
-- ih.setup({
--     only_current_line = true,
--     eol = {
--         right_align = true,
--         right_align_padding = 10,
--     }
-- })

-- lsp_lines includes a better solution, disabled to avoid double diags.
vim.diagnostic.config({
  virtual_text = false,
})

local capabilities = vim.lsp.protocol.make_client_capabilities()
capabilities = require("cmp_nvim_lsp").default_capabilities(capabilities)
capabilities.textDocument.completion.completionItem.snippetSupport = true

lsp_config.terraformls.setup({})

lsp_config.java_language_server.setup({
    cmd = { "/usr/bin/java-language-server" },
})

lsp_config.gopls.setup({
    capabilities = capabilities,
    -- on_attach = function(c, b)
    --     ih.on_attach(c, b)
    -- end,
    settings = {
        gopls = {
            experimentalPostfixCompletions = true,
            analyses = {
                fieldalignment = true,
                nilness = true,
                unusedwrite = true,
                unusedparams = true,
                unusedvariable = true,
                shadow = true,
                unreachable = true,
                bools = true,
                useany = true,
            },
            staticcheck = true,
            usePlaceholders = true,
            codelenses = {
                gc_details = true,
            },
            semanticTokens = true,
            hints = {
                assignVariableTypes = true,
                compositeLiteralFields = true,
                compositeLiteralTypes = true,
                constantValues = true,
                functionTypeParameters = true,
                parameterNames = true,
                rangeVariableTypes = true,
            },
        },
    },
})

-- lsp_config.rust_analyzer.setup{
--     capabilities = capabilities,
--     settings = {
--         ["rust-analyzer"] = {
--             assist = {
--                 importGranularity = "module",
--                 importPrefix = "by_self",
--             },
--             cargo = {
--                 loadOutDirsFromCheck = true
--             },
--             procMacro = {
--                 enable = true
--             },
--         }
--     }
-- }

local opts = {
    tools = { -- rust-tools options
        -- Automatically set inlay hints (type hints)
        autoSetHints = true,

        -- Whether to show hover actions inside the hover window
        -- This overrides the default hover handler
        -- hover_with_actions = true,

        runnables = {
            -- whether to use telescope for selection menu or not
            use_telescope = true,

            -- rest of the opts are forwarded to telescope
        },

        debuggables = {
            -- whether to use telescope for selection menu or not
            use_telescope = true,

            -- rest of the opts are forwarded to telescope
        },

        -- These apply to the default RustSetInlayHints command
        inlay_hints = {

            -- Only show inlay hints for the current line
            only_current_line = false,

            -- Event which triggers a refersh of the inlay hints.
            -- You can make this "CursorMoved" or "CursorMoved,CursorMovedI" but
            -- not that this may cause  higher CPU usage.
            -- This option is only respected when only_current_line and
            -- autoSetHints both are true.
            only_current_line_autocmd = "CursorHold",

            -- wheter to show parameter hints with the inlay hints or not
            show_parameter_hints = true,

            -- prefix for parameter hints
            parameter_hints_prefix = "<- ",

            -- prefix for all the other hints (type, chaining)
            other_hints_prefix = "=> ",

            -- whether to align to the length of the longest line in the file
            max_len_align = false,

            -- padding from the left if max_len_align is true
            max_len_align_padding = 1,

            -- whether to align to the extreme right or not
            right_align = false,

            -- padding from the right if right_align is true
            right_align_padding = 7,

            -- The color of the hints
            highlight = "Comment",
        },

        hover_actions = {
            -- the border that is used for the hover window
            -- see vim.api.nvim_open_win()
            border = {
                { "╭", "FloatBorder" },
                { "─", "FloatBorder" },
                { "╮", "FloatBorder" },
                { "│", "FloatBorder" },
                { "╯", "FloatBorder" },
                { "─", "FloatBorder" },
                { "╰", "FloatBorder" },
                { "│", "FloatBorder" },
            },

            -- whether the hover action window gets automatically focused
            auto_focus = false,
        },
    },

    -- all the opts to send to nvim-lspconfig
    -- these override the defaults set by rust-tools.nvim
    -- see https://github.com/neovim/nvim-lspconfig/blob/master/CONFIG.md#rust_analyzer
    server = {}, -- rust-analyer options
}

require("rust-tools").setup(opts)

lsp_config.angularls.setup({
    capabilities = capabilities,
})

lsp_config.yamlls.setup({
    capabilities = capabilities,
})

lsp_config.metals.setup({
    capabilities = capabilities,
    settings = {
        metals = {
            superMethodLensesEnabled = true,
            showInferredType = true,
            showImplicitArguments = true,
        },
    },
})

lsp_config.dartls.setup({})

lsp_config.texlab.setup({})

lsp_config.ccls.setup({
    capabilities = capabilities,
})

lsp_config.sumneko_lua.setup({
    cmd = { "/usr/bin/lua-language-server" },
    -- on_attach = function(c, b)
    --     ih.on_attach(c, b)
    -- end,
    capabilities = capabilities,
    settings = {
        Lua = {
            hint = {
                enable = true,
            },
            runtime = {
                -- Tell the language server which version of Lua you're using (most likely LuaJIT in the case of Neovim)
                version = "LuaJIT",
                -- Setup your lua path
                path = vim.split(package.path, ";"),
            },
            diagnostics = {
                -- Get the language server to recognize the `vim` global
                globals = { "vim" },
            },
            workspace = {
                -- Make the server aware of Neovim runtime files
                library = {
                    [vim.fn.expand("$VIMRUNTIME/lua")] = true,
                    [vim.fn.expand("$VIMRUNTIME/lua/vim/lsp")] = true,
                },
            },
            -- Do not send telemetry data containing a randomized but unique identifier
            telemetry = {
                enable = false,
            },
        },
    },
})

local eslint = {
    lintCommand = "eslint_d -f unix --stdin --stdin-filename ${INPUT}",
    lintStdin = true,
    lintFormats = { "%f:%l:%c: %m" },
    lintIgnoreExitCode = true,
    formatCommand = "eslint_d --fix-to-stdout --stdin --stdin-filename=${INPUT}",
    formatStdin = true,
}

lsp_config.efm.setup({
    init_options = { documentFormatting = true },
    filetypes = { "javascript", "typescript" },
    root_dir = function(fname)
        return lsp_config.util.root_pattern("tsconfig.json")(fname)
            or lsp_config.util.root_pattern(".eslintrc.js", ".git")(fname)
    end,
    settings = {
        rootMarkers = { ".eslintrc.js", ".git/" },
        languages = {
            javascript = { eslint },
            typescript = { eslint },
        },
    },
})

-- vim.lsp.handlers["textDocument/publishDiagnostics"] = vim.lsp.with(
-- vim.lsp.diagnostic.on_publish_diagnostics, {
--     underline = true,
--     virtual_text = {
--         prefix = " » ",
--         spacing = 8,
--         severity_limit = "Warning",
--     },
--     signs = true,
--     update_in_insert = true,
-- }
-- )

-- require('lsp_signature').on_attach({
--     bind = true, -- This is mandatory, otherwise border config won't get registered.
--     -- If you want to hook lspsaga or other signature handler, pls set to false
--     doc_lines = 4, -- will show two lines of comment/doc(if there are more than two lines in doc, will be truncated);
--     -- set to 0 if you do not want any API comments be shown
--     -- This setting only take effect in insert mode, it does not affect signature help in normal
--     -- mode
--
--     hint_enable = true, -- virtual hint enable
--     hint_prefix = "↑ ",  -- Panda for parameter
--     hint_scheme = "String",
--     use_lspsaga = false,  -- set to true if you want to use lspsaga popup
--     handler_opts = {
--         border = "shadow"   -- double, single, shadow, none
--     },
--     decorator = {"`", "`"}  -- decoractor can be `decorator = {"***", "***"}`  `decorator = {"**", "**"}` `decorator = {"**_", "_**"}`
--     -- `decorator = {"*", "*"} see markdown help for more details
--     -- <u></u> ~ ~ does not supported by nvim
-- })

if vim.g.snippets ~= "luasnip" then
    return
end

local ls = require("luasnip")
require("snippets")

ls.config.set_config({
    history = true,
    updateevents = "TextChanged,TextChangedI",
})

-- create snippet
-- s(context, nodes, condition, ...)
local snippet = ls.s
local snippet_from_nodes = ls.sn

-- This a choice snippet. You can move through with <c-e> (in my config)
-- tbl_snip {
--   trig = "c",
--   t { "-- this has a choice: " },
--   c(1, { t {"hello"}, t {"world"}, }),
--   i(0),
-- }
local c = ls.c -- choice node

local f = ls.f -- function node
local i = ls.i -- insert node
local t = ls.t -- text node
local d = ls.d -- dynamic node

local str = function(text)
    return t({ text })
end

local newline = function(text)
    return t({ "", text })
end

local str_snip = function(trig, expanded)
    return ls.parser.parse_snippet({ trig = trig }, expanded)
end

local tbl_snip = function(t)
    return snippet({ trig = t.trig, dscr = t.desc }, { unpack(t) })
end

local function char_count_same(c1, c2)
    local line = vim.api.nvim_get_current_line()
    local _, ct1 = string.gsub(line, c1, "")
    local _, ct2 = string.gsub(line, c2, "")
    return ct1 == ct2
end

local function neg(fn, ...)
    return not fn(...)
end

-- {{{ Go stuff
local ts_locals = require("nvim-treesitter.locals")
local ts_utils = require("nvim-treesitter.ts_utils")

local get_node_text = vim.treesitter.get_node_text

vim.treesitter.set_query(
    "go",
    "LuaSnip_Result",
    [[
  [
    (method_declaration result: (*) @id)
    (function_declaration result: (*) @id)
    (func_literal result: (*) @id)
  ]
]]
)

local transform = function(text, info)
    if text == "int" then
        return str("0")
    elseif text == "error" then
        if info then
            info.index = info.index + 1

            return c(info.index, {
                str(string.format('errors.Wrap(%s, "%s")', info.err_name, info.func_name)),
                str(info.err_name),
            })
        else
            return str("err")
        end
    elseif text == "bool" then
        return str("false")
    elseif string.find(text, "*", 1, true) then
        return str("nil")
    end

    return str(text)
end

local handlers = {
    ["parameter_list"] = function(node, info)
        local result = {}

        local count = node:named_child_count()
        for i = 0, count - 1 do
            table.insert(result, transform(get_node_text(node:named_child(i), 0), info))
            if i ~= count - 1 then
                table.insert(result, t({ ", " }))
            end
        end

        return result
    end,

    ["type_identifier"] = function(node, info)
        local text = get_node_text(node, 0)
        return { transform(text, info) }
    end,
}

local function go_result_type(info)
    local cursor_node = ts_utils.get_node_at_cursor()
    local scope = ts_locals.get_scope_tree(cursor_node, 0)

    local function_node
    for _, v in ipairs(scope) do
        if v:type() == "function_declaration" or v:type() == "method_declaration" or v:type() == "func_literal" then
            function_node = v
            break
        end
    end

    local query = vim.treesitter.get_query("go", "LuaSnip_Result")
    for id, node in query:iter_captures(function_node, 0) do
        if handlers[node:type()] then
            return handlers[node:type()](node, info)
        end
    end
end

-- }}}
local shortcut = function(val)
    if type(val) == "string" then
        return { t({ val }), i(0) }
    end

    if type(val) == "table" then
        for k, v in ipairs(val) do
            if type(v) == "string" then
                val[k] = t({ v })
            end
        end
    end

    return val
end

local make = function(tbl)
    local result = {}
    for k, v in pairs(tbl) do
        table.insert(result, (snippet({ trig = k, desc = v.desc }, shortcut(v))))
    end

    return result
end

local same = function(index)
    return f(function(args)
        return args[1]
    end, { index })
end

local snippets = {}
local go_ret_vals = function(args, old_state)
    local info = { index = 0, err_name = args[1][1], func_name = args[2][1] }
    return snippet_from_nodes(nil, go_result_type(info))
end

--stylua: ignore
snippets.go = make {
    main = {
        t { "func main() {", "\t" },
        i(0),
        t { "", "}" },
    },

    ef = {
        i(1, { "val" }),
        str ", err := ",
        i(2, { "f" }),
        str "(",
        i(3),
        str ")",
        i(0),
    },

    efi = {
        i(1, { "val" }),
        ", ",
        i(2, { "err" }),
        " := ",
        i(3, { "f" }),
        "(",
        i(4),
        ")",
        t { "", "if " },
        same(2),
        t { " != nil {", "\treturn " },
        d(5, go_ret_vals, { 2, 3 }),
        t { "", "}" },
        i(0),
    },

    -- TODO: Fix this up so that it actually uses the tree sitter thing
    ie = { "if err != nil {", "\treturn err", i(0), "}" },
}

vim.cmd([[augroup lsp]])
vim.cmd([[au!]])
vim.cmd([[au FileType scala,sbt lua require("metals").initialize_or_attach({})]])
vim.cmd([[augroup end]])

local metals_config = require("metals").bare_config()
metals_config.settings = {
    showImplicitArguments = true,
}

require('dap-go').setup()
